@@ -11,40 +11,55 @@
 # limitations under the License.
 
 import abc
-import collections
+import dataclasses
 import locale
 import logging
 import shlex
 import subprocess
 import urllib.parse
+from typing import TYPE_CHECKING, Any, Optional, Union
+
+if TYPE_CHECKING:
+    import testinfra.host
 
 logger = logging.getLogger("testinfra")
 
-HostSpec = collections.namedtuple("HostSpec", ["name", "port", "user", "password"])
+
+@dataclasses.dataclass
+class HostSpec:
+    name: str
+    port: Optional[str]
+    user: Optional[str]
+    password: Optional[str]
 
 
+@dataclasses.dataclass
 class CommandResult:
-    def __init__(
-        self,
-        backend,
-        exit_status,
-        command,
-        stdout_bytes,
-        stderr_bytes,
-        stdout=None,
-        stderr=None,
-    ):
-        self.exit_status = exit_status
-        self._stdout_bytes = stdout_bytes
-        self._stderr_bytes = stderr_bytes
-        self._stdout = stdout
-        self._stderr = stderr
-        self.command = command
-        self._backend = backend
-        super().__init__()
+    """Object that encapsulates all returned details of the command execution.
+
+    Example:
+
+    >>> cmd = host.run("ls -l /etc/passwd")
+    >>> cmd.rc
+    0
+    >>> cmd.stdout
+    '-rw-r--r-- 1 root root 1790 Feb 11 00:28 /etc/passwd\\n'
+    >>> cmd.stderr
+    ''
+    >>> cmd.succeeded
+    True
+    >>> cmd.failed
+    False
+    """
+
+    backend: "BaseBackend"
+    exit_status: int
+    command: bytes
+    _stdout: Union[str, bytes]
+    _stderr: Union[str, bytes]
 
     @property
-    def succeeded(self):
+    def succeeded(self) -> bool:
         """Returns whether the command was successful
 
         >>> host.run("true").succeeded
@@ -53,7 +68,7 @@ class CommandResult:
         return self.exit_status == 0
 
     @property
-    def failed(self):
+    def failed(self) -> bool:
         """Returns whether the command failed
 
         >>> host.run("false").failed
@@ -62,7 +77,7 @@ class CommandResult:
         return self.exit_status != 0
 
     @property
-    def rc(self):
+    def rc(self) -> int:
         """Gets the returncode of a command
 
         >>> host.run("true").rc
@@ -71,38 +86,48 @@ class CommandResult:
         return self.exit_status
 
     @property
-    def stdout(self):
-        if self._stdout is None:
-            self._stdout = self._backend.decode(self._stdout_bytes)
+    def stdout(self) -> str:
+        """Gets standard output (stdout) stream of an executed command
+
+        >>> host.run("mkdir -v new_directory").stdout
+        mkdir: created directory 'new_directory'
+        """
+        if isinstance(self._stdout, bytes):
+            return self.backend.decode(self._stdout)
         return self._stdout
 
     @property
-    def stderr(self):
-        if self._stderr is None:
-            self._stderr = self._backend.decode(self._stderr_bytes)
+    def stderr(self) -> str:
+        """Gets standard error (stderr) stream of an executed command
+
+        >>> host.run("mkdir new_directory").stderr
+        mkdir: cannot create directory 'new_directory': File exists
+        """
+        if isinstance(self._stderr, bytes):
+            return self.backend.decode(self._stderr)
         return self._stderr
 
     @property
-    def stdout_bytes(self):
-        if self._stdout_bytes is None:
-            self._stdout_bytes = self._backend.encode(self._stdout)
-        return self._stdout_bytes
+    def stdout_bytes(self) -> bytes:
+        """Gets standard output (stdout) stream of an executed command as bytes
+
+        >>> host.run("mkdir -v new_directory").stdout_bytes
+        b"mkdir: created directory 'new_directory'"
+        """
+        if isinstance(self._stdout, str):
+            return self.backend.encode(self._stdout)
+        return self._stdout
 
     @property
-    def stderr_bytes(self):
-        if self._stderr_bytes is None:
-            self._stderr_bytes = self._backend.encode(self._stderr)
-        return self._stderr_bytes
+    def stderr_bytes(self) -> bytes:
+        """Gets standard error (stderr) stream of an executed command as bytes
 
-    def __repr__(self):
-        return (
-            "CommandResult(command={!r}, exit_status={}, stdout={!r}, " "stderr={!r})"
-        ).format(
-            self.command,
-            self.exit_status,
-            self._stdout_bytes or self._stdout,
-            self._stderr_bytes or self._stderr,
-        )
+        >>> host.run("mkdir new_directory").stderr_bytes
+        b"mkdir: cannot create directory 'new_directory': File exists"
+        """
+        if isinstance(self._stderr, str):
+            return self.backend.encode(self._stderr)
+        return self._stderr
 
 
 class BaseBackend(metaclass=abc.ABCMeta):
@@ -112,26 +137,33 @@ class BaseBackend(metaclass=abc.ABCMeta):
     HAS_RUN_ANSIBLE = False
     NAME: str
 
-    def __init__(self, hostname, sudo=False, sudo_user=None, *args, **kwargs):
-        self._encoding = None
-        self._host = None
+    def __init__(
+        self,
+        hostname: str,
+        sudo: bool = False,
+        sudo_user: Optional[str] = None,
+        *args: Any,
+        **kwargs: Any,
+    ):
+        self._encoding: Optional[str] = None
+        self._host: Optional["testinfra.host.Host"] = None
         self.hostname = hostname
         self.sudo = sudo
         self.sudo_user = sudo_user
         super().__init__()
 
-    def set_host(self, host):
+    def set_host(self, host: "testinfra.host.Host") -> None:
         self._host = host
 
     @classmethod
-    def get_connection_type(cls):
+    def get_connection_type(cls) -> str:
         """Return the connection backend used as string.
 
         Can be local, paramiko, ssh, docker, salt or ansible
         """
         return cls.NAME
 
-    def get_hostname(self):
+    def get_hostname(self) -> str:
         """Return the hostname (for testinfra) of the remote or local system
 
 
@@ -158,11 +190,11 @@ class BaseBackend(metaclass=abc.ABCMeta):
         """
         return self.hostname
 
-    def get_pytest_id(self):
+    def get_pytest_id(self) -> str:
         return self.get_connection_type() + "://" + self.get_hostname()
 
     @classmethod
-    def get_hosts(cls, host, **kwargs):
+    def get_hosts(cls, host: str, **kwargs: Any) -> list[str]:
         if host is None:
             raise RuntimeError(
                 "One or more hosts is required with the {} backend".format(
@@ -172,41 +204,41 @@ class BaseBackend(metaclass=abc.ABCMeta):
         return [host]
 
     @staticmethod
-    def quote(command, *args):
+    def quote(command: str, *args: str) -> str:
         if args:
             return command % tuple(shlex.quote(a) for a in args)  # noqa: S001
         return command
 
-    def get_sudo_command(self, command, sudo_user):
+    def get_sudo_command(self, command: str, sudo_user: Optional[str]) -> str:
         if sudo_user is None:
             return self.quote("sudo /bin/sh -c %s", command)
         return self.quote("sudo -u %s /bin/sh -c %s", sudo_user, command)
 
-    def get_command(self, command, *args):
+    def get_command(self, command: str, *args: str) -> str:
         command = self.quote(command, *args)
         if self.sudo:
             command = self.get_sudo_command(command, self.sudo_user)
         return command
 
-    def run(self, command, *args, **kwargs):
+    def run(self, command: str, *args: str, **kwargs: Any) -> CommandResult:
         raise NotImplementedError
 
-    def run_local(self, command, *args):
+    def run_local(self, command: str, *args: str) -> CommandResult:
         command = self.quote(command, *args)
-        command = self.encode(command)
+        cmd = self.encode(command)
         p = subprocess.Popen(
-            command,
+            cmd,
             shell=True,
             stdin=subprocess.PIPE,
             stdout=subprocess.PIPE,
             stderr=subprocess.PIPE,
         )
         stdout, stderr = p.communicate()
-        result = self.result(p.returncode, command, stdout, stderr)
+        result = self.result(p.returncode, cmd, stdout, stderr)
         return result
 
     @staticmethod
-    def parse_hostspec(hostspec):
+    def parse_hostspec(hostspec: str) -> HostSpec:
         name = hostspec
         port = None
         user = None
@@ -238,7 +270,7 @@ class BaseBackend(metaclass=abc.ABCMeta):
         return HostSpec(name, port, user, password)
 
     @staticmethod
-    def parse_containerspec(containerspec):
+    def parse_containerspec(containerspec: str) -> tuple[str, Optional[str]]:
         name = containerspec
         user = None
         if "@" in name:
@@ -266,24 +298,32 @@ class BaseBackend(metaclass=abc.ABCMeta):
         return encoding
 
     @property
-    def encoding(self):
+    def encoding(self) -> str:
         if self._encoding is None:
             self._encoding = self.get_encoding()
         return self._encoding
 
-    def decode(self, data):
+    def decode(self, data: bytes) -> str:
         try:
             return data.decode("ascii")
         except UnicodeDecodeError:
             return data.decode(self.encoding)
 
-    def encode(self, data):
+    def encode(self, data: str) -> bytes:
         try:
             return data.encode("ascii")
         except UnicodeEncodeError:
             return data.encode(self.encoding)
 
-    def result(self, *args, **kwargs):
-        result = CommandResult(self, *args, **kwargs)
+    def result(
+        self, rc: int, cmd: bytes, stdout: Union[str, bytes], stderr: Union[str, bytes]
+    ) -> CommandResult:
+        result = CommandResult(
+            backend=self,
+            exit_status=rc,
+            command=cmd,
+            _stdout=stdout,
+            _stderr=stderr,
+        )
         logger.debug("RUN %s", result)
         return result
